EXEC [__AddStoredProcInfo]
    /* StoredProc_Name         */ 'NpmPackages_CreateOrUpdatePackage',
    /* Internal_Indicator      */ 'N',
    /* ReturnType_Name         */ 'void',
    /* DataTableNames_Csv      */ NULL,
    /* OutputPropertyNames_Csv */ NULL,
    /* Description_Text        */ 'Creates a new npm package or updates an existing one.'
GO

IF OBJECT_ID('[NpmPackages_CreateOrUpdatePackage]') IS NOT NULL DROP PROCEDURE [NpmPackages_CreateOrUpdatePackage]
GO

SET QUOTED_IDENTIFIER ON
GO

CREATE PROCEDURE [NpmPackages_CreateOrUpdatePackage]
(
	@Feed_Id INT,
	@Package_Name NVARCHAR(200),
	@Scope_Name VARCHAR(200),
	@Version_Text VARCHAR(200),
	@PackageJson_Bytes VARBINARY(MAX),
	@Published_Date DATETIME,
	@Tag_Name NVARCHAR(200),
	@PackageHash_Bytes BINARY(20),
	@Cached_Indicator YNINDICATOR,
	@Package_Size BIGINT,
	@SHA512Hash_Bytes BINARY(64)
)
AS
BEGIN

SET NOCOUNT ON

BEGIN TRY
	BEGIN TRANSACTION

		SET @Scope_Name = COALESCE(@Scope_Name, '')

		DECLARE @Package_Id INT
		SET @Package_Id = (SELECT [Package_Id] FROM [NpmPackages] WITH (UPDLOCK) WHERE [Feed_Id] = @Feed_Id AND [Scope_Name] = @Scope_Name AND [Package_Name] = @Package_Name)

		IF @Package_Id IS NULL
		BEGIN

			INSERT INTO [NpmPackages]
			(
				[Feed_Id],
				[Package_Name],
				[Scope_Name],
				[Download_Count]
			)
			VALUES
			(
				@Feed_Id,
				@Package_Name,
				@Scope_Name,
				0
			)

			SET @Package_Id = SCOPE_IDENTITY()

		END

		IF EXISTS(SELECT * FROM [NpmPackageVersions] WITH (UPDLOCK) WHERE [Package_Id] = @Package_Id AND [Version_Text] = @Version_Text)
		BEGIN

			UPDATE [NpmPackageVersions]
			   SET [PackageJson_Bytes] = COALESCE(@PackageJson_Bytes, [PackageJson_Bytes]),
			       [Published_Date] = COALESCE(@Published_Date, [Published_Date]),
				   [PackageHash_Bytes] = COALESCE(@PackageHash_Bytes, [PackageHash_Bytes]),
				   [Package_Size] = COALESCE(@Package_Size, [Package_Size]),
				   [Cached_Indicator] = COALESCE(@Cached_Indicator, [Cached_Indicator]),
				   [SHA512Hash_Bytes] = COALESCE(@SHA512Hash_Bytes, [SHA512Hash_Bytes])
			 WHERE [Package_Id] = @Package_Id
			   AND [Version_Text] = @Version_Text

			IF COALESCE(dbo.Session_GetUserName(), 'UNKNOWN') NOT IN ('SYSTEM', 'UNKNOWN')
   				EXEC [Events_RaiseEvent] 'PKGMDF', @Feed_Id, 'Package_Id', @Package_Id, 'Package_Version', @Version_Text

		END
		ELSE
		BEGIN

			INSERT INTO [NpmPackageVersions]
			(
				[Package_Id],
				[Version_Text],
				[PackageJson_Bytes],
				[Download_Count],
				[Published_Date],
				[PackageHash_Bytes],
				[Package_Size],
				[Cached_Indicator],
				[SHA512Hash_Bytes]
			)
			VALUES
			(
				@Package_Id,
				@Version_Text,
				@PackageJson_Bytes,
				0,
				COALESCE(@Published_Date, GETUTCDATE()),
				@PackageHash_Bytes,
				@Package_Size,
				COALESCE(@Cached_Indicator, 'N'),
				@SHA512Hash_Bytes
			)

			EXEC [Events_RaiseEvent] 'PKGADD', @Feed_Id, 'Package_Id', @Package_Id, 'Package_Version', @Version_Text

			--ensure that there is always a latest when adding a new package
			IF NOT EXISTS(SELECT * FROM [NpmPackageTags] WITH (UPDLOCK) WHERE [Package_Id] = @Package_Id AND [Tag_Name] = 'latest')
			BEGIN
				INSERT INTO [NpmPackageTags]
				(
					[Package_Id],
					[Tag_Name],
					[Tag_Version]
				)
				VALUES
				(
					@Package_Id,
					'latest',
					@Version_Text
				)
			END

		END

		IF @Tag_Name IS NOT NULL
		BEGIN

			IF EXISTS(SELECT * FROM [NpmPackageTags] WITH (UPDLOCK) WHERE [Package_Id] = @Package_Id AND [Tag_Name] = @Tag_Name)
			BEGIN

				UPDATE [NpmPackageTags]
				   SET [Tag_Version] = @Version_Text
				 WHERE [Package_Id] = @Package_Id
				   AND [Tag_Name] = @Tag_Name

			END
			ELSE
			BEGIN

				INSERT INTO [NpmPackageTags]
				(
					[Package_Id],
					[Tag_Name],
					[Tag_Version]
				)
				VALUES
				(
					@Package_Id,
					@Tag_Name,
					@Version_Text
				)

			END

		END

	COMMIT
END TRY BEGIN CATCH
	IF XACT_STATE()<>0 ROLLBACK
	EXEC [HandleError]
END CATCH

END
GO

GRANT EXECUTE ON [NpmPackages_CreateOrUpdatePackage] TO [ProGetUser_Role]
GO